package eu.eudat.dao.entities;

import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Root;

import org.hibernate.query.Query;

import eu.eudat.dao.JpaDao;
import eu.eudat.entities.Project;
import eu.eudat.entities.responses.IDLabelPair;
import eu.eudat.models.project.ProjectTableRequest;
import org.springframework.stereotype.Component;

@Component("projectDao")
public class ProjectDaoImpl extends JpaDao<Project, UUID> implements ProjectDao {

	public Project loadDetails(Project t) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public List<UUID> listAllIDs() {
		String queryString = "SELECT project.id FROM Project project";
		TypedQuery<UUID> typedQuery = entityManager.createQuery(queryString, UUID.class);
		return typedQuery.getResultList();
	}
	
	
	@Override
	public List<IDLabelPair> listAllIDsLabels() {
		String queryString = "SELECT project.id, project.label FROM Project project";
		Query query = (Query) entityManager.createQuery(queryString);
		List<Object[]> rows = query.list();
		return rows.stream().map(row -> {
			return new IDLabelPair(row[0].toString(), row[1].toString());
		})
		.collect(Collectors.toList());
	}
	
	
	public List<Project> getProjectsOfUser(String userID){
		
		String queryString = "select p from Project p where p.creationUser.id=:userid and project.status >= 0";
		TypedQuery<Project> typedQuery = entityManager.createQuery(queryString, Project.class);
		typedQuery.setParameter("userid", UUID.fromString(userID));
		try {
			return typedQuery.getResultList();
		}
		catch(Exception ex) { //no need to distinguish between eu.eudat.exceptions for the moment
			ex.printStackTrace();
			return null;
		}
		
	}

	@Override
	public List<Project> getWithCriteria(ProjectTableRequest projectTableRequest) {
		CriteriaBuilder criteriaBuilder = entityManager.getCriteriaBuilder();
		CriteriaQuery<Project> criteriaQuery = criteriaBuilder .createQuery(Project.class);	
		Root<Project> root = criteriaQuery.from(Project.class);
		TypedQuery<Project> typedQuery = entityManager.createQuery(criteriaQuery);
		typedQuery.setFirstResult(projectTableRequest.getOffset()); 
		typedQuery.setMaxResults(projectTableRequest.getLength());
		return typedQuery.getResultList();
	}

	
	
	

}
