import { Component, ViewChild, OnInit, AfterViewInit } from "@angular/core";
import { MatPaginator, MatSort, MatSnackBar } from "@angular/material";
import { Router } from "@angular/router";
import { TranslateService } from "@ngx-translate/core";
import { DataSource } from "@angular/cdk/table";
import { Observable } from "rxjs/Observable";
import { DataTableRequest } from "../models/data-table/DataTableRequest";
import { SnackBarNotificationComponent } from "../shared/components/notificaiton/snack-bar-notification.component";
import { DataManagementPlanService } from "../services/data-management-plan/data-management-plan.service";
import { DataManagementPlanCriteriaComponent } from "../shared/components/criteria/data-management-plan/dmp-criteria.component";
import { DataManagementPlanCriteria } from "../models/criteria/data-management-plan/DataManagementPlanCriteria";
import { DataManagementPlanListingModel } from "../models/data-managemnt-plans/DataManagementPlanListingModel";



@Component({
  selector: 'app-dmp-listing-component',
  templateUrl: 'dmp-listing.component.html',
  styleUrls: ['./dmp-listing.component.css'],
  providers: [DataManagementPlanService]
})
export class DataManagementPlanListingComponent implements OnInit, AfterViewInit {

  @ViewChild(MatPaginator) _paginator: MatPaginator;
  @ViewChild(MatSort) sort: MatSort;
  @ViewChild(DataManagementPlanCriteriaComponent) criteria: DataManagementPlanCriteriaComponent;

  dataSource: DataManagementPlanDataSource | null;
  displayedColumns: String[] = ['name', 'abbreviation', 'start', 'end'];

  constructor(
    private projectService: DataManagementPlanService,
    private router: Router,
    private languageService: TranslateService,
    public snackBar: MatSnackBar,
  ) {

  }

  ngOnInit() {

  }

  ngAfterViewInit() {
    setTimeout(() => {
      this.criteria.setRefreshCallback(() => this.refresh());
      this.criteria.setCriteria(this.getDefaultCriteria());
      this.criteria.controlModified();
    });
  }

  refresh() {
    this.dataSource = new DataManagementPlanDataSource(this.projectService, this._paginator, this.sort, this.languageService, this.snackBar, this.criteria, );
  }

  rowClick(rowId: String) {
    this.router.navigate(['/forms/' + rowId]);
  }

  getDefaultCriteria(): DataManagementPlanCriteria {
    const defaultCriteria = new DataManagementPlanCriteria();
    return defaultCriteria;
  }

  // canShowOwner(): boolean {
  //   const principal: Principal = this.authentication.current();
  //   if (principal) {
  //     const principalRoles = principal.appRoles;
  //     for (let i = 0; i < principalRoles.length; i++) {
  //       if (principalRoles[i] === Principal.AppRole.Admin || principalRoles[i] === Principal.AppRole.BudgetManager) {
  //         return true;
  //       }
  //     }
  //   }
  //   return false;
  // }
}

export class DataManagementPlanDataSource extends DataSource<DataManagementPlanListingModel> {

  totalCount = 0;
  isLoadingResults = false;

  constructor(
    private _service: DataManagementPlanService,
    private _paginator: MatPaginator,
    private _sort: MatSort,
    private _languageService: TranslateService,
    private _snackBar: MatSnackBar,
    private _criteria: DataManagementPlanCriteriaComponent
  ) {
    super();

    //this._paginator.page.subscribe((pageEvent: PageEvent) => {
    //    this.store.dispatch(new LoadPhotosRequestAction(pageEvent.pageIndex, pageEvent.pageSize))
    //})
  }

  connect(): Observable<DataManagementPlanListingModel[]> {
    const displayDataChanges = [
      this._paginator.page
      //this._sort.matSortChange
    ];

    // If the user changes the sort order, reset back to the first page.
    //this._sort.matSortChange.subscribe(() => {
    //    this._paginator.pageIndex = 0;
    //})

    return Observable.merge(...displayDataChanges)
      .startWith(null)
      .switchMap(() => {
        setTimeout(() => {
          this.isLoadingResults = true;
        });
        const startIndex = this._paginator.pageIndex * this._paginator.pageSize;
        const request = new DataTableRequest(startIndex, this._paginator.pageSize);
        request.projectCriteria = this._criteria.getFormData();
        return this._service.getPaged(request);
      })
      .catch((error: any) => {
        this._snackBar.openFromComponent(SnackBarNotificationComponent, {
          data: { message: 'GENERAL.SNACK-BAR.FORMS-BAD-REQUEST', language: this._languageService },
          duration: 3000,
          extraClasses: ['snackbar-warning']
        });
        this._criteria.onCallbackError(error);
        return Observable.of(null);
      })
      .map(result => {
        setTimeout(() => {
          this.isLoadingResults = false;
        });
        return result;
      })
      .map(result => {
        if (!result) { return []; }
        if (this._paginator.pageIndex === 0) { this.totalCount = result.totalCount; }
        return result.data;
      });
  }

  disconnect() {
    // No-op
  }
}
