import { Serializable } from "../Serializable";
import { ValidationContext } from "../../utilities/validators/ValidationContext";
import { FormGroup, FormBuilder, FormControl, Validators } from "@angular/forms";
import { BackendErrorValidator } from "../../utilities/validators/BackendErrorValidator";
import { BaseErrorModel } from "../error/BaseErrorModel";

export class ProjectModel implements Serializable<ProjectModel> {
	public id: String;
	public label: String;
	public abbreviation: String;
	public reference: String;
	public uri: String;
	public status: String;
	public startDate: Date;
	public endDate: Date;
	public description: String;

	public errorModel: BaseErrorModel = new BaseErrorModel();

	fromJSONObject(item: any): ProjectModel {
		this.id = item.id;
		this.label = item.label;
        this.abbreviation = item.abbreviation;
        this.reference = item.reference;
        this.uri = item.uri;
        this.status = item.status;
        this.startDate = item.startDate;
        this.endDate = item.endDate;
        this.description = item.description;

		return this;
	}

	buildForm(context: ValidationContext = null, disabled: boolean = false): FormGroup {
		if (context == null) { context = this.createValidationContext(); }
		
		const formGroup = new FormBuilder().group({
			label: [{ value: this.label, disabled: disabled }, context.getValidation('label').validators],
			abbreviation: [{ value: this.abbreviation, disabled: disabled }, context.getValidation('abbreviation').validators],
			reference: [{ value: this.reference, disabled: disabled }],
			uri: [{ value: this.uri, disabled: disabled }, context.getValidation('uri').validators],
			description: [{ value: this.description, disabled: disabled }, context.getValidation('description').validators],
			startDate: [{ value: this.startDate, disabled: disabled }, context.getValidation('startDate').validators],
			endDate: [{ value: this.endDate, disabled: disabled }, context.getValidation('endDate').validators]
		});

		return formGroup;
	}

	createValidationContext(): ValidationContext {
		const baseContext: ValidationContext = new ValidationContext();
		baseContext.validation.push({ key: 'label', validators: [Validators.required, BackendErrorValidator(this.errorModel, 'label')] });
		baseContext.validation.push({ key: 'abbreviation', validators: [Validators.required, BackendErrorValidator(this.errorModel, 'abbreviation')] });
		baseContext.validation.push({ key: 'uri', validators: [Validators.required, BackendErrorValidator(this.errorModel, 'uri')] });
		baseContext.validation.push({ key: 'description', validators: [Validators.required, BackendErrorValidator(this.errorModel, 'description')] });
		baseContext.validation.push({ key: 'startDate', validators: [Validators.required, BackendErrorValidator(this.errorModel, 'startDate')] });
		baseContext.validation.push({ key: 'endDate', validators: [Validators.required, BackendErrorValidator(this.errorModel, 'endDate')] });

		return baseContext;
	}
}