import { Component, ViewChild, OnInit, AfterViewInit } from "@angular/core";
import { MatPaginator, MatSort, MatSnackBar } from "@angular/material";
import { Router, ActivatedRoute, Params } from "@angular/router";
import { TranslateService } from "@ngx-translate/core";
import { DataSource } from "@angular/cdk/table";
import { Observable } from "rxjs/Observable";
import { DataManagementPlanService } from "../../services/data-management-plan/data-management-plan.service";
import { ProjectModel } from "../../models/projects/ProjectModel";
import { ProjectService } from "../../services/project/project.service";
import { JsonSerializer } from "../../utilities/JsonSerializer";
import { FormGroup } from "@angular/forms";



@Component({
  selector: 'app-project-editor-component',
  templateUrl: 'project-editor.component.html',
  styleUrls: ['./project-editor.component.css'],
  providers: [ProjectService]
})
export class ProjectEditorComponent implements AfterViewInit {

  isNew = true;
  project: ProjectModel;
  formGroup: FormGroup = null;

  constructor(
    private projectService: ProjectService,
    private route: ActivatedRoute,
  ) {

  }

  ngAfterViewInit() {
    this.route.params.subscribe((params: Params) => {
      const itemId = params['id'];

      if (itemId != null) {
        this.isNew = false;
        this.projectService.getSingle(itemId).map(data => data as ProjectModel)
          .subscribe(data => {
            this.project = new JsonSerializer<ProjectModel>().fromJSONObject(data, ProjectModel);
            this.formGroup = this.project.buildForm();
          });
      } else {
        this.project = new ProjectModel();
        this.formGroup = this.project.buildForm();
      }
    });
  }
}